/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.common.attached_item;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.bescheid.SentInfoMapper;
import de.ozgcloud.command.CommandTestFactory;
import de.ozgcloud.document.DocumentService;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;

class AttachedItemMapperTest {

	@Spy
	@InjectMocks
	private AttachedItemMapper mapper = Mappers.getMapper(AttachedItemMapper.class);

	@Mock
	private GrpcObjectMapper grpcObjectMapper;
	@Mock
	private SentInfoMapper sentInfoMapper;

	@Nested
	class TestMapFromVorgangAttachedItem {

		@BeforeEach
		void init() {
			when(grpcObjectMapper.mapFromGrpc(any())).thenReturn(AttachedItemTestFactory.createDocumentItem());
		}

		@Test
		void shouldMapFromVorgangAttachedItem() {
			var grpcVorgangAttachedItem = GrpcVorgangAttachedItemTestFactory.createBuilder().setItemName(DocumentService.DOCUMENT_ITEM_NAME).build();

			var result = mapper.mapFromVorgangAttachedItem(grpcVorgangAttachedItem);

			assertThat(result).usingRecursiveComparison().isEqualTo(AttachedItemTestFactory.createDocument());
		}
	}

	@Nested
	class TestAsMap {

		@Test
		void shouldSetVorgangId() {
			var result = mapper.toMap(AttachedItemTestFactory.createBescheid());

			assertThat(result).containsEntry(AttachedItem.PROPERTY_VORGANG_ID, CommandTestFactory.VORGANG_ID);
		}

		@Test
		void shouldSetClient() {
			var result = mapper.toMap(AttachedItemTestFactory.createBescheid());

			assertThat(result).containsEntry(AttachedItem.PROPERTY_CLIENT, AttachedItemTestFactory.CLIENT);
		}

		@Test
		void shouldSetItemName() {
			var result = mapper.toMap(AttachedItemTestFactory.createBescheid());

			assertThat(result).containsEntry(AttachedItem.PROPERTY_ITEM_NAME, AttachedItemService.BESCHEID_ITEM_NAME);
		}

		@Test
		void shouldSetItem() {
			var item = Map.<String, Object>of("key", "value");
			var attachedItem = AttachedItemTestFactory.createBescheidBuilder().clearItem().item(item).build();

			var result = mapper.toMap(attachedItem);

			assertThat(result).containsEntry(AttachedItem.PROPERTY_ITEM, item);
		}
	}
}