/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.clientattribute;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.vorgang.vorgang.VorgangTestFactory;

class ClientAttributeServiceTest {

	@InjectMocks
	private ClientAttributeService service;

	@Mock
	private ClientAttributeRepository repository;

	@Nested
	class SetAttribute {

		@Test
		void shouldCallRepository() {
			var attribute = ClientAttributeTestFactory.create();

			service.set(VorgangTestFactory.ID, attribute);

			verify(repository).set(VorgangTestFactory.ID, attribute);
		}

		@Test
		void shouldReturnPersistedDocument() {
			var attribute = ClientAttributeTestFactory.create();
			when(repository.set(anyString(), any())).thenReturn(attribute);

			var persisted = service.set(VorgangTestFactory.ID, attribute);

			assertThat(persisted).isSameAs(attribute);
		}
	}

	@Nested
	class TestUpdateAttribute {

		@BeforeEach
		void init() {
			when(repository.exists(any())).thenReturn(true);
		}

		@Test
		void shouldCheckExisting() {
			service.update(VorgangTestFactory.ID, ClientAttributeTestFactory.create());

			verify(repository).exists(ClientAttributeIdentificatorTestFactory.create());
		}

		@Test
		void shouldThrowExceptionIfNOTExists() {
			when(repository.exists(any())).thenReturn(false);

			var attribute = ClientAttributeTestFactory.create();
			assertThatThrownBy(() -> service.update(VorgangTestFactory.ID, attribute))
					.isInstanceOf(ClientAttributeNotExistingException.class);
		}

		@Test
		void callRepository() {
			var attribute = ClientAttributeTestFactory.create();

			service.update(VorgangTestFactory.ID, attribute);

			verify(repository).update(VorgangTestFactory.ID, attribute);
		}
	}

	@Nested
	class DeleteAttribute {

		@Test
		void shouldCallRepository() {
			var id = ClientAttributeIdentificatorTestFactory.create();

			service.delete(id);

			verify(repository).delete(id);
		}
	}

}
